-- TODO: Priorities for Tiger's Fury and Berserk?

if not PriorityQueue then return end

if select(2, UnitClass('player')) ~= "DRUID" then
	return
end

local L = LibStub("AceLocale-3.0"):GetLocale("PriorityQueue")
local C = PriorityQueue.constants -- Defined in PriorityQueue LUA no locale needed.
local latestEclipse = nil

C["GCD Spell"] = GetSpellInfo(1126)

-- Cat Spells
C["Swipe"], _, C["Swipe Icon"] = GetSpellInfo(779)
C["Mangle (Cat)"], _, C["Mangle (Cat) Icon"] = GetSpellInfo(33876)
C["Savage Roar"], _, C["Savage Roar Icon"] = GetSpellInfo(52610)
C["Rake"], _, C["Rake Icon"] = GetSpellInfo(1822)
C["Shred"], _, C["Shred Icon"] = GetSpellInfo(5221)
C["Rip"], _, C["Rip Icon"] = GetSpellInfo(1079)
C["Ferocious Bite"], _, C["Ferocious Bite Icon"] = GetSpellInfo(22568)

-- Moonkin Spells
C["Wrath"], _, C["Wrath Icon"] = GetSpellInfo(5176)
C["Moonfire"], _, C["Moonfire Icon"] = GetSpellInfo(8921)
C["Faerie Fire"], _, C["Faerie Fire Icon"] = GetSpellInfo(770)
C["Starfire"], _, C["Starfire Icon"] = GetSpellInfo(2912)
C["Insect Swarm"], _, C["Insect Swarm Icon"] = GetSpellInfo(24974)
C["Starfall"], _, C["Starfall Icon"] = GetSpellInfo(53201)
C["Innervate"], _, C["Innervate Icon"] = GetSpellInfo(29166)

-- buffs:
C["Solar Eclipse"] = GetSpellInfo(48517)
C["Lunar Eclipse"] = GetSpellInfo(48518)

-- Debuffs:
C["Mangle (Bear)"] = GetSpellInfo(33878)
C["Trauma"] = GetSpellInfo(46857)

function PriorityQueue:UNIT_MANA()
	if self.db.char.priority.show and InCombatLockdown() then
		self:SetNextPriority()
	end
end

function PriorityQueue:UNIT_RAGE()
	if self.db.char.priority.show and InCombatLockdown() then
		self:SetNextPriority()
	end
end

function PriorityQueue:UNIT_ENERGY()
	if self.db.char.priority.show and InCombatLockdown() then
		self:SetNextPriority()
	end
end

---------------------------
-- Druid Priorities
---------------------------

function PriorityQueue:GetDruidPriorityDefaults()
	self:RegisterEvent("UNIT_MANA")
	self:RegisterEvent("UNIT_RAGE")
	self:RegisterEvent("UNIT_ENERGY")
	-- need a function to work out spec for now fixed values
	self.presets.priorities.name["Cat"] =  L["Cat"]
	self.presets.priorities.list["Cat"] = { "4cp_sr", "mc", "rake", "5cp_rip", "4cp_fbr", "sh", "none", "none", "none", "none" }
	self.presets.priorities.name["Moonkin"] =  L["Moonkin"]
	self.presets.priorities.list["Moonkin"] = { "inv", "ff", "is", "mf", "sfall", "sfire", "wr", "none", "none", "none" }
	self.presets.default = "Cat"
end

-- Cat Spells

PriorityQueue.priorityTable.name["sw"] = C["Swipe"]
PriorityQueue.priorityTable.icon["sw"] = C["Swipe Icon"]
PriorityQueue.priorityTable.test["sw"] = 
	function () 
		return PriorityQueue:SpellAvailable(C["Swipe"]) 
	end 

PriorityQueue.priorityTable.name["mc"] = C["Mangle (Cat)"]
PriorityQueue.priorityTable.icon["mc"] = C["Mangle (Cat) Icon"]
PriorityQueue.priorityTable.test["mc"] = 
	function ()
		-- DEBUG: Issues with SpellAvailable() method for Mangle... Hard-coded to assume availability.
		local MangleCat, MangleCatDuration = PriorityQueue:GetDebuffInfo(C["Mangle (Cat)"])
		local MangleBear, MangleBearDuration = PriorityQueue:GetDebuffInfo(C["Mangle (Bear)"])
		local Trauma, TraumaDuration = PriorityQueue:GetDebuffInfo(C["Trauma"])
		
		return (not MangleCat or MangleCatDuration <= 1) and (not MangleBear or MangleBearDuration <= 1) and (not Trauma or TraumaDuration <= 1)
		--return (not debuff or duration <= 1) and PriorityQueue:SpellAvailable(C["Mangle (Cat)"])
		--return PriorityQueue:SpellAvailable(C["Mangle (Cat)"])
		
	end

PriorityQueue.priorityTable.name["rake"] = C["Rake"]
PriorityQueue.priorityTable.icon["rake"] = C["Rake Icon"]
PriorityQueue.priorityTable.test["rake"] = 
	function ()
		return not PriorityQueue:GetDebuffInfo(C["Rake"]) and PriorityQueue:SpellAvailable(C["Rake"])
	end

PriorityQueue.priorityTable.name["sh"] = C["Shred"]
PriorityQueue.priorityTable.icon["sh"] = C["Shred Icon"]
PriorityQueue.priorityTable.test["sh"] = 
	function () 
		return PriorityQueue:SpellAvailable(C["Shred"])
	end

PriorityQueue.priorityTable.name["4cp_sr"] = L["4CP Savage Roar"]
PriorityQueue.priorityTable.icon["4cp_sr"] = C["Savage Roar Icon"]
PriorityQueue.priorityTable.test["4cp_sr"] = 
	function ()
		local buff, duration = PriorityQueue:GetBuffInfo(C["Savage Roar"])
		
		return (not buff or duration < 4) and GetComboPoints("player", "target") >= 4 and PriorityQueue:SpellAvailable(C["Savage Roar"])
	end

PriorityQueue.priorityTable.name["5cp_rip"] = L["5CP Rip"]
PriorityQueue.priorityTable.icon["5cp_rip"] = C["Rip Icon"]
PriorityQueue.priorityTable.test["5cp_rip"] = 
	function ()
		return not PriorityQueue:GetBuffInfo(C["Rip"]) and PriorityQueue:SpellAvailable(C["Rip"]) and GetComboPoints("player", "target") >= 5
	end

PriorityQueue.priorityTable.name["4cp_rip"] = L["4CP Rip"]
PriorityQueue.priorityTable.icon["4cp_rip"] = C["Rip Icon"]
PriorityQueue.priorityTable.test["4cp_rip"] = 
	function ()
		return not PriorityQueue:GetBuffInfo(C["Rip"]) and PriorityQueue:SpellAvailable(C["Rip"]) and GetComboPoints("player", "target") >= 4
	end

PriorityQueue.priorityTable.name["4cp_fbr"] = L["4CP Ferocious Bite w/Rip"]
PriorityQueue.priorityTable.icon["4cp_fbr"] = C["Ferocious Bite Icon"]
PriorityQueue.priorityTable.test["4cp_fbr"] = 
	function () 
		return PriorityQueue:SpellAvailable(C["Ferocious Bite"]) and GetComboPoints("player", "target") >= 4 and PriorityQueue:GetDebuffInfo(C["Rip"])
	end

PriorityQueue.priorityTable.name["5cp_fb"] = L["5CP Ferocious Bite"]
PriorityQueue.priorityTable.icon["5cp_fb"] = C["Ferocious Bite Icon"]
PriorityQueue.priorityTable.test["5cp_fb"] = 
	function () 
		return PriorityQueue:SpellAvailable(C["Ferocious Bite"]) and GetComboPoints("player", "target") >= 5
	end
	
-- Moonkin Spells

function PriorityQueue:CheckLastEclipse()
	if PriorityQueue:GetBuffInfo(C["Solar Eclipse"]) then
		latestEclipse = C["Solar Eclipse"]
	elseif PriorityQueue:GetBuffInfo(C["Lunar Eclipse"]) then
		latestEclipse = C["Lunar Eclipse"]
	end
end	

PriorityQueue.priorityTable.name["wr"] = C["Wrath"]
PriorityQueue.priorityTable.icon["wr"] = C["Wrath Icon"]
PriorityQueue.priorityTable.test["wr"] = 
	function () 
		PriorityQueue:CheckLastEclipse()
		return PriorityQueue:SpellAvailable(C["Wrath"]) and (latestEclipse == nil or latestEclipse == C["Solar Eclipse"])
	end 

PriorityQueue.priorityTable.name["sfire"] = C["Starfire"]
PriorityQueue.priorityTable.icon["sfire"] = C["Starfire Icon"]
PriorityQueue.priorityTable.test["sfire"] = 
	function () 
		PriorityQueue:CheckLastEclipse()
		return PriorityQueue:SpellAvailable(C["Starfire"]) and latestEclipse == C["Lunar Eclipse"]
	end 

PriorityQueue.priorityTable.name["sfall"] = C["Starfall"]
PriorityQueue.priorityTable.icon["sfall"] = C["Starfall Icon"]
PriorityQueue.priorityTable.test["sfall"] = 
	function () 
		return PriorityQueue:SpellAvailable(C["Starfall"])
	end 

PriorityQueue.priorityTable.name["inv"] = C["Innervate"]
PriorityQueue.priorityTable.icon["inv"] = C["Innervate Icon"]
PriorityQueue.priorityTable.test["inv"] = 
	function () 
		-- use innervate if available and current mana < 10% of max mana
		return PriorityQueue:SpellAvailable(C["Innervate"]) and UnitPower("player") <= UnitPowerMax("player") / 10 
	end 

PriorityQueue.priorityTable.name["mf"] = C["Moonfire"]
PriorityQueue.priorityTable.icon["mf"] = C["Moonfire Icon"]
PriorityQueue.priorityTable.test["mf"] = 
	function () 
		return PriorityQueue:SpellAvailable(C["Moonfire"]) and not PriorityQueue:GetDebuffInfo(C["Moonfire"])
	end 

PriorityQueue.priorityTable.name["is"] = C["Insect Swarm"]
PriorityQueue.priorityTable.icon["is"] = C["Insect Swarm Icon"]
PriorityQueue.priorityTable.test["is"] = 
	function () 
		return PriorityQueue:SpellAvailable(C["Insect Swarm"]) and not PriorityQueue:GetDebuffInfo(C["Insect Swarm"])
	end 

PriorityQueue.priorityTable.name["ff"] = C["Faerie Fire"]
PriorityQueue.priorityTable.icon["ff"] = C["Faerie Fire Icon"]
PriorityQueue.priorityTable.test["ff"] = 
	function () 
		return PriorityQueue:SpellAvailable(C["Faerie Fire"]) and not PriorityQueue:GetDebuffInfo(C["Faerie Fire"])
	end 

