if not PriorityQueue then return end

if select(2, UnitClass('player')) ~= "HUNTER" then
	return
end

local L = LibStub("AceLocale-3.0"):GetLocale("PriorityQueue")
local C = PriorityQueue.constants -- Defined in PriorityQueue LUA no locale needed.

C["GCD Spell"] = GetSpellInfo(1130)
C["Kill Shot"], _, C["Kill Shot Icon"] = GetSpellInfo(53351)
C["Black Arrow"], _, C["Black Arrow Icon"] = GetSpellInfo(3674)
C["Aimed Shot"], _, C["Aimed Shot Icon"] = GetSpellInfo(19434)
C["Arcane Shot"], _, C["Arcane Shot Icon"] = GetSpellInfo(49045)
C["Multi Shot"], _, C["Multi Shot Icon"] = GetSpellInfo(2643)
C["Serpent Sting"], _, C["Serpent Sting Icon"] = GetSpellInfo(1978)
C["Steady Shot"], _, C["Steady Shot Icon"] = GetSpellInfo(56641)
C["Explosive Shot"], _, C["Explosive Shot Icon"] = GetSpellInfo(53301)
C["Chimera Shot"], _, C["Chimera Shot Icon"] = GetSpellInfo(53209)
C["Feign Death"], _, C["Feign Death Icon"] = GetSpellInfo(5384)
C["Bestial Wrath"], _, C["Bestial Wrath Icon"] = GetSpellInfo(19574)

function PriorityQueue:UNIT_MANA()
	if self.db.char.priority.show and InCombatLockdown() then
		self:SetNextPriority()
	end
end

---------------------------
-- Hunter Priorities
---------------------------

function PriorityQueue:GetHunterPriorityDefaults()
	self:RegisterEvent("UNIT_MANA")
	self.db.char.priority.dropthreat.name = C["Feign Death"]
	self.db.char.priority.dropthreat.icon = C["Feign Death Icon"]

	-- need a function to work out spec for now fixed values
	self.presets.priorities.name["Survival"] =  L["Survival"]
	self.presets.priorities.name["Marksman"] =  L["Marksman"]
	self.presets.priorities.name["Beast Master"] =  L["Beast Master"]
	self.presets.priorities.list["Survival"] = { "ks", "ba", "es_ll", "es", "as", "sps", "ss", "none", "none", "none" }
	self.presets.priorities.list["Marksman"] = { "sps", "cs",  "arcs",  "as", "ms", "ss", "none", "none", "none", "none" }
	self.presets.priorities.list["Beast Master"] = { "ks", "sps", "bw",  "arcs", "ss", "none", "none", "none", "none", "none" }
	self.presets.default = "Survival"
end

PriorityQueue.priorityTable.name["bw"] = C["Bestial Wrath"]
PriorityQueue.priorityTable.icon["bw"] = C["Bestial Wrath Icon"]
PriorityQueue.priorityTable.test["bw"] = 
	function () 
		return PriorityQueue:SpellAvailable(C["Bestial Wrath"])
	end 

PriorityQueue.priorityTable.name["cs"] = C["Chimera Shot"]
PriorityQueue.priorityTable.icon["cs"] = C["Chimera Shot Icon"]
PriorityQueue.priorityTable.test["cs"] = 
	function () 
		return PriorityQueue:SpellAvailable(C["Chimera Shot"])
	end 

PriorityQueue.priorityTable.name["arcs"] = C["Arcane Shot"]
PriorityQueue.priorityTable.icon["arcs"] = C["Arcane Shot Icon"]
PriorityQueue.priorityTable.test["arcs"] = 
	function () 
		return PriorityQueue:SpellAvailable(C["Arcane Shot"])
	end 

PriorityQueue.priorityTable.name["es"] = C["Explosive Shot"]
PriorityQueue.priorityTable.icon["es"] = C["Explosive Shot Icon"]
PriorityQueue.priorityTable.test["es"] = 
	function () 
		return PriorityQueue:SpellAvailable(C["Explosive Shot"]) and not PriorityQueue:GetDebuffInfo(C["Explosive Shot"])
	end 

PriorityQueue.priorityTable.name["es_ll"] = L["Explosive Shot if LL"]
PriorityQueue.priorityTable.icon["es_ll"] = C["Explosive Shot Icon"]
PriorityQueue.priorityTable.test["es_ll"] =  
	function () 
		return PriorityQueue:GetBuffInfo(C["Lock and Load"]) and PriorityQueue:SpellAvailable(C["Explosive Shot"]) and not PriorityQueue:GetDebuffInfo(C["Explosive Shot"])
	end 

PriorityQueue.priorityTable.name["ks"] = C["Kill Shot"]
PriorityQueue.priorityTable.icon["ks"] = C["Kill Shot Icon"]
PriorityQueue.priorityTable.test["ks"] =  
	function () 
		local unithealth = 100 * UnitHealth("target") / UnitHealthMax("target")
		if unithealth <= 20 then -- only kill shot if target health < 20%
			return PriorityQueue:SpellAvailable(C["Kill Shot"])
		end
		return false
	end 
	
PriorityQueue.priorityTable.name["sps"] = C["Serpent Sting"]
PriorityQueue.priorityTable.icon["sps"] = C["Serpent Sting Icon"]
PriorityQueue.priorityTable.test["sps"] = 
	function () 
		return not PriorityQueue:GetDebuffInfo(C["Serpent Sting"]) and PriorityQueue:SpellAvailable(C["Serpent Sting"])
	end 
	

PriorityQueue.priorityTable.name["ba"] = C["Black Arrow"]
PriorityQueue.priorityTable.icon["ba"] = C["Black Arrow Icon"]
PriorityQueue.priorityTable.test["ba"] = 
	function () 
		return PriorityQueue:SpellAvailable(C["Black Arrow"])
	end 

PriorityQueue.priorityTable.name["as"] = C["Aimed Shot"]
PriorityQueue.priorityTable.icon["as"] = C["Aimed Shot Icon"]
PriorityQueue.priorityTable.test["as"] = 
	function () 
		return PriorityQueue:SpellAvailable(C["Aimed Shot"])
	end 

PriorityQueue.priorityTable.name["ms"] = C["Multi Shot"]
PriorityQueue.priorityTable.icon["ms"] = C["Multi Shot Icon"]
PriorityQueue.priorityTable.test["ms"] = 
	function () 
		return PriorityQueue:SpellAvailable(C["Multi Shot"])
	end 

PriorityQueue.priorityTable.name["ss"] = C["Steady Shot"]
PriorityQueue.priorityTable.icon["ss"] = C["Steady Shot Icon"]
PriorityQueue.priorityTable.test["ss"] = 
	function () 
		return PriorityQueue:SpellAvailable(C["Steady Shot"])
	end 
