if not PriorityQueue then return end

if select(2, UnitClass('player')) ~= "ROGUE" then
	return
end

local L = LibStub("AceLocale-3.0"):GetLocale("PriorityQueue")
local C = PriorityQueue.constants -- Defined in PriorityQueue LUA no locale needed.

C["GCD Spell"] = GetSpellInfo(1752)
C["Sinister Strike"], _, C["Sinister Strike Icon"] = GetSpellInfo(1752)
C["Mutilate"], _, C["Mutilate Icon"] = GetSpellInfo(1329)
C["Rupture"], _, C["Rupture Icon"] = GetSpellInfo(1943)
C["Slice and Dice"], _, C["Slice and Dice Icon"] = GetSpellInfo(5171)
C["Hemorrhage"], _, C["Hemorrhage Icon"] = GetSpellInfo(48660)
C["Envenom"], _, C["Envenom Icon"] = GetSpellInfo(57993)
C["Eviscerate"], _, C["Eviscerate Icon"] = GetSpellInfo(48668)
C["Hunger For Blood"], _, C["Hunger For Blood Icon"] = GetSpellInfo(51662)

-- Spells needed to check for debuffs
-- TODO - expand list to     *  Blood Frenzy, Deep Wounds, Rend, Rupture, Garrote,
--      Lacerate, Pounce, Rip, Rake (Druid), Rake (Hunter Cat), Savage Rend

C["Deep Wounds"], _, C["Deep Wounds Icon"] = GetSpellInfo(12867)
C["Rend"], _, C["Rend Icon"] = GetSpellInfo(47465)

C["Deadly Poison"], _, C["Deadly Poison Icon"] = GetSpellInfo(43232)

function PriorityQueue:UNIT_ENERGY()
	if self.db.char.priority.show and InCombatLockdown() then
		self:SetNextPriority()
	end
end

---------------------------
-- Rogue Priorities
---------------------------

function PriorityQueue:GetRoguePriorityDefaults()
	self:RegisterEvent("UNIT_ENERGY")
	-- need a function to work out spec for now fixed values
	self.presets.priorities.name["Combat"] =  L["Combat"]
	self.presets.priorities.name["Mutilate"] =  L["Mutilate"]
	self.presets.priorities.list["Combat"] = { "snd", "rup",  "evi",  "ss", "none", "none", "none", "none", "none", "none" }
	self.presets.priorities.list["Mutilate"] = { "hfb",  "snd",  "rup", "env", "mut", "none", "none", "none", "none", "none"  }
	self.presets.default = "Mutilate"
end

PriorityQueue.priorityTable.name["ss"] = C["Sinister Strike"]
PriorityQueue.priorityTable.icon["ss"] = C["Sinister Strike Icon"]
PriorityQueue.priorityTable.test["ss"] = 
	function () 
		return PriorityQueue:SpellAvailable(C["Sinister Strike"]) 
	end 

PriorityQueue.priorityTable.name["mut"] = C["Mutilate"]
PriorityQueue.priorityTable.icon["mut"] = C["Mutilate Icon"]
PriorityQueue.priorityTable.test["mut"] = 
	function () 
		return PriorityQueue:SpellAvailable(C["Mutilate"]) 
	end 

PriorityQueue.priorityTable.name["rup"] = C["Rupture"]
PriorityQueue.priorityTable.icon["rup"] = C["Rupture Icon"]
PriorityQueue.priorityTable.test["rup"] = 
	function () 
		local debuff, duration = PriorityQueue:GetDebuffInfo(C["Rupture"])
		return UnitPower("player") >= 25 
			and GetComboPoints("player") >= 4
			and (not debuff or duration < 4) 
			and PriorityQueue:SpellAvailable(C["Rupture"])
			and PriorityQueue:GetBuffInfo(C["Slice and Dice"]) 
	end 

PriorityQueue.priorityTable.name["snd"] = C["Slice and Dice"]
PriorityQueue.priorityTable.icon["snd"] = C["Slice and Dice Icon"]
PriorityQueue.priorityTable.test["snd"] = 
	function () 
		local debuff, duration = PriorityQueue:GetBuffInfo(C["Slice and Dice"])
		return UnitPower("player") >= 25
			and GetComboPoints("player") >= 4
			and (not debuff or duration < 6) 
			and PriorityQueue:SpellAvailable(C["Slice and Dice"]) 
	end

PriorityQueue.priorityTable.name["evi"] = C["Eviscerate"]
PriorityQueue.priorityTable.icon["evi"] = C["Eviscerate Icon"]
PriorityQueue.priorityTable.test["evi"] = 
	function () 
		return GetComboPoints("player") >= 4
			and PriorityQueue:SpellAvailable(C["Eviscerate"]) 
	end 
	
	
	-- Add in check for deadly poison
PriorityQueue.priorityTable.name["env"] = C["Envenom"]
PriorityQueue.priorityTable.icon["env"] = C["Envenom Icon"]
PriorityQueue.priorityTable.test["env"] = 
	function () 
		return GetComboPoints("player") >= 4
			and PriorityQueue:GetBuffInfo(C["Slice and Dice"])
			and PriorityQueue:GetDebuffInfo(C["Rupture"])
			and PriorityQueue:GetDebuffInfo(C["Deadly Poison"])
			and PriorityQueue:SpellAvailable(C["Envenom"]) 
	end 		

-- TODO - expand list to     *  Blood Frenzy, Deep Wounds, Rend, Rupture, Garrote,
--      Lacerate, Pounce, Rip, Rake (Druid), Rake (Hunter Cat), Savage Rend

-- True if a bleed effect is on unitID, false otherwise	
-- unitID defaults to target
function PriorityQueue:Bleed(unitID)
	   unitID = unitID or "target"
	   
	   return PriorityQueue:GetDebuffInfo(C["Rupture"], true)
	   	 or   PriorityQueue:GetDebuffInfo(C["Deep Wounds"], true)
	   	 or	  PriorityQueue:GetDebuffInfo(C["Rend"], true)
end
	
PriorityQueue.priorityTable.name["hfb"] = C["Hunger For Blood"]
PriorityQueue.priorityTable.icon["hfb"] = C["Hunger For Blood Icon"]
PriorityQueue.priorityTable.test["hfb"] = 
	function () 
		local buff, duration = PriorityQueue:GetBuffInfo(C["Hunger For Blood"])
		return 		PriorityQueue:Bleed() 
				and UnitPower("player") >= 15 
				and (not buff or duration < 4) 
				and PriorityQueue:SpellAvailable(C["Hunger For Blood"]) 
	end	 

