if not PriorityQueue then return end

if select(2, UnitClass('player')) ~= "WARRIOR" then
	return
end

local L = LibStub("AceLocale-3.0"):GetLocale("PriorityQueue")
local C = PriorityQueue.constants -- Defined in PriorityQueue LUA no locale needed.

C["GCD Spell"] = GetSpellInfo(6673)

-- Abilities:
---- Common:
C["Execute"], _, C["Execute Icon"] = GetSpellInfo(5308)
C["Heroic Strike"], _, C["Heroic Strike Icon"] = GetSpellInfo(78)
C["Slam"], _, C["Slam Icon"] = GetSpellInfo(1464)

---- Prot:
C["Shield Slam"], _, C["Shield Slam Icon"] = GetSpellInfo(23922)
C["Revenge"], _, C["Revenge Icon"] = GetSpellInfo(6572)
C["Devastate"], _, C["Devastate Icon"] = GetSpellInfo(20243)
C["Concussion Blow"], _, C["Concussion Blow"] = GetSpellInfo(12809)
C["Shockwave"], _, C["Shockwave Icon"] = GetSpellInfo(46968)
-- TODO: Add Thunder Clap, Judgements of the Just, and Frost Fever for TC priority
-- TODO: Add Demoralizing Shout, Demoralizing Roar, and Vindication for Demo Shout priority
-- TODO: Add Expose Armor?

---- Fury:
C["Bloodthirst"], _, C["Bloodthirst Icon"] = GetSpellInfo(23881)
C["Whirlwind"], _, C["Whirlwind Icon"] = GetSpellInfo(1680)

---- Arms:
-- Mortal strike, Overpower, Bladestorm, Rend...
C["Mortal Strike"], _, C["Mortal Strike Icon"] = GetSpellInfo(12294)
C["Overpower"], _, C["Overpower Icon"] = GetSpellInfo(7384)
C["Rend"], _, C["Rend Icon"] = GetSpellInfo(772)
C["Bladestorm"], _, C["Bladestorm Icon"] = GetSpellInfo(46924)

-- Procs/Buffs:
C["Slam!"] = GetSpellInfo(46916)
C["Sudden Death"] = GetSpellInfo(52437)
C["Taste for Blood"] = GetSpellInfo(60503)
C["Sword and Board"] = GetSpellInfo(50227)
C["Sunder Armor"] = GetSpellInfo(58567)

function PriorityQueue:UNIT_RAGE()
	if self.db.char.priority.show and InCombatLockdown() then
		self:SetNextPriority()
	end
end

---------------------------
-- Warrior Priorities
---------------------------

function PriorityQueue:GetWarriorPriorityDefaults()
	self:RegisterEvent("UNIT_RAGE")
	-- need a function to work out spec for now fixed values
	self.presets.priorities.name["Arms"] =  L["Arms"]
	self.presets.priorities.list["Arms"] = { "rend", "ms",  "op",  "sd_ex", "hs80", "slam", "none", "none", "none", "none" }
	self.presets.priorities.name["Fury"] =  L["Fury"]
	self.presets.priorities.list["Fury"] = { "ww", "slam_i", "bt",  "ex", "hs60", "none", "none", "none", "none", "none" }
	self.presets.priorities.name["Protection"] =  L["Protection"]
	self.presets.priorities.list["Protection"] = { "hs60", "ss", "rev", "dev5s", "sw", "dev", "none", "none", "none", "none" }
	self.presets.default = "Protection"
end

-- Common priorities:
PriorityQueue.priorityTable.name["ex"] = C["Execute"]
PriorityQueue.priorityTable.icon["ex"] = C["Execute Icon"]
PriorityQueue.priorityTable.test["ex"] = 
	function () 
		local unithealth = 100 * UnitHealth("target") / UnitHealthMax("target")
		if unithealth <= 20 then
			return PriorityQueue:SpellAvailable(C["Execute"])
		end
		return false
	end

PriorityQueue.priorityTable.name["hs"] = C["Heroic Strike"]
PriorityQueue.priorityTable.icon["hs"] = C["Heroic Strike Icon"]
PriorityQueue.priorityTable.test["hs"] = 
	function () 
		return PriorityQueue:SpellAvailable(C["Heroic Strike"]) 
	end

PriorityQueue.priorityTable.name["hs60"] = L["Heroic Strike @ 60+ rage"]
PriorityQueue.priorityTable.icon["hs60"] = C["Heroic Strike Icon"]
PriorityQueue.priorityTable.test["hs60"] = 
	function () 
		return not PriorityQueue.events.nextSwing and PriorityQueue:SpellAvailable(C["Heroic Strike"]) and UnitPower("player") >= 60
	end

PriorityQueue.priorityTable.name["hs80"] = L["Heroic Strike @ 80+ rage"]
PriorityQueue.priorityTable.icon["hs80"] = C["Heroic Strike Icon"]
PriorityQueue.priorityTable.test["hs80"] = 
	function () 
		return not PriorityQueue.events.nextSwing and PriorityQueue:SpellAvailable(C["Heroic Strike"]) and UnitPower("player") >= 80
	end

-- Prot-specific:
PriorityQueue.priorityTable.name["ss"] = C["Shield Slam"]
PriorityQueue.priorityTable.icon["ss"] = C["Shield Slam Icon"]
PriorityQueue.priorityTable.test["ss"] = 
	function () 
		return PriorityQueue:SpellAvailable(C["Shield Slam"]) 
	end 

PriorityQueue.priorityTable.name["snb"] = L["Shield Slam w/SnB"]
PriorityQueue.priorityTable.icon["snb"] = C["Shield Slam Icon"]
PriorityQueue.priorityTable.test["snb"] = 
	function () 
		return PriorityQueue:GetBuffInfo(C["Sword and Board"]) and PriorityQueue:SpellAvailable(C["Shield Slam"]) 
	end 

-- TODO: Figure out why this insists on showing up when it is outside its usable window, on cooldown, etc.
PriorityQueue.priorityTable.name["rev"] = C["Revenge"]
PriorityQueue.priorityTable.icon["rev"] = C["Revenge Icon"]
PriorityQueue.priorityTable.test["rev"] = 
	function ()
		local returnVal = false
		
		if PriorityQueue:SpellAvailable(C["Revenge"]) then
			-- Calculate time since last dodge/block/parry...
			-- Assuming 5-second availability window for Revenge... Please change to the appropriate value if you can confirm.
			local dodgeparryWindowRemaining = PriorityQueue.events.dodgeparryTime + 5.0 - GetTime()
			local blockWindowRemaining = PriorityQueue.events.blockTime + 5.0 - GetTime()
		
			returnVal = (PriorityQueue.events.dodgeparry and (dodgeparryWindowRemaining > 0)) or (PriorityQueue.events.block and (blockWindowRemaining > 0))	
		end
		
		return  returnVal 
	end 

PriorityQueue.priorityTable.name["dev"] = C["Devastate"]
PriorityQueue.priorityTable.icon["dev"] = C["Devastate Icon"]
PriorityQueue.priorityTable.test["dev"] = 
	function () 
		return PriorityQueue:SpellAvailable(C["Devastate"]) 
	end

PriorityQueue.priorityTable.name["dev5s"] = L["Devastate if < 5 Sunders"]
PriorityQueue.priorityTable.icon["dev5s"] = C["Devastate Icon"]
PriorityQueue.priorityTable.test["dev5s"] = 
	function ()
		_, duration, stacks = PriorityQueue:GetDebuffInfo(C["Sunder Armor"], true)
		return ((stacks < 5) or (duration < 3)) and PriorityQueue:SpellAvailable(C["Devastate"])
	end 

PriorityQueue.priorityTable.name["cb"] = C["Concussion Blow"]
PriorityQueue.priorityTable.icon["cb"] = C["Concussion Blow Icon"]
PriorityQueue.priorityTable.test["cb"] = 
	function () 
		return PriorityQueue:SpellAvailable(C["Concussion Blow"]) 
	end 
	
PriorityQueue.priorityTable.name["sw"] = C["Shockwave"]
PriorityQueue.priorityTable.icon["sw"] = C["Shockwave Icon"]
PriorityQueue.priorityTable.test["sw"] = 
	function () 
		return PriorityQueue:SpellAvailable(C["Shockwave"]) 
	end
	
-- TODO: Thunder Clap on boss if !debuff
-- TODO: Demo Shout on boss if !debuff

-- Fury-specific:
PriorityQueue.priorityTable.name["bt"] = C["Bloodthirst"]
PriorityQueue.priorityTable.icon["bt"] = C["Bloodthirst Icon"]
PriorityQueue.priorityTable.test["bt"] = 
	function () 
		return PriorityQueue:SpellAvailable(C["Bloodthirst"]) 
	end 

PriorityQueue.priorityTable.name["ww"] = C["Whirlwind"]
PriorityQueue.priorityTable.icon["ww"] = C["Whirlwind Icon"]
PriorityQueue.priorityTable.test["ww"] = 
	function () 
		return PriorityQueue:SpellAvailable(C["Whirlwind"]) 
	end

PriorityQueue.priorityTable.name["slam_i"] = L["Slam (Instant)"]
PriorityQueue.priorityTable.icon["slam_i"] = C["Slam Icon"]
PriorityQueue.priorityTable.test["slam_i"] = 
	function () 
		return PriorityQueue:SpellAvailable(C["Slam"]) and PriorityQueue:GetBuffInfo(C["Slam!"])
	end
	
-- Arms-specific:

PriorityQueue.priorityTable.name["slam"] = C["Slam"]
PriorityQueue.priorityTable.icon["slam"] = C["Slam Icon"]
PriorityQueue.priorityTable.test["slam"] = 
	function () 
		return not PriorityQueue.events.nextSwing and PriorityQueue:SpellAvailable(C["Slam"])
	end

PriorityQueue.priorityTable.name["rend"] = C["Rend"]
PriorityQueue.priorityTable.icon["rend"] = C["Rend Icon"]
PriorityQueue.priorityTable.test["rend"] = 
	function () 
		return PriorityQueue:SpellAvailable(C["Rend"]) and not PriorityQueue:GetDebuffInfo(C["Rend"])
	end

PriorityQueue.priorityTable.name["op"] = C["Overpower"]
PriorityQueue.priorityTable.icon["op"] = C["Overpower Icon"]
PriorityQueue.priorityTable.test["op"] = 
	function () 
		return PriorityQueue:SpellAvailable(C["Overpower"]) and (PriorityQueue.events.target_dodged or PriorityQueue:GetBuffInfo(C["Taste for Blood"]))
	end
	
PriorityQueue.priorityTable.name["ms"] = C["Mortal Strike"]
PriorityQueue.priorityTable.icon["ms"] = C["Mortal Strike Icon"]
PriorityQueue.priorityTable.test["ms"] = 
	function () 
		return PriorityQueue:SpellAvailable(C["Mortal Strike"])
	end
	
PriorityQueue.priorityTable.name["bs"] = C["Bladestorm"]
PriorityQueue.priorityTable.icon["bs"] = C["Bladestorm Icon"]
PriorityQueue.priorityTable.test["bs"] = 
	function () 
		return PriorityQueue:SpellAvailable(C["Bladestorm"])
	end
	
PriorityQueue.priorityTable.name["sd_ex"] = L["Sudden Death Execute"]
PriorityQueue.priorityTable.icon["sd_ex"] = C["Execute Icon"]
PriorityQueue.priorityTable.test["sd_ex"] = 
	function ()
		return PriorityQueue:SpellAvailable(C["Execute"]) and PriorityQueue:GetBuffInfo(C["Sudden Death"])
	end